#AutoIt3Wrapper_Au3Check_Parameters= -d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
#include-once
#include <IE.au3>

; #INDEX# =======================================================================================================================
; Title .........: ArrayEx
; AutoIt Version : 3.3.8.1
; Language ......: English
; Description ...: Functions that assist with array 1D and 2D.
; Author(s) .....: Tlem
; Notes .........: Most of the code is inspired by array's functions and SmOke_N code from this link :
; 				   http://www.autoitscript.com/forum/topic/26544-multi-dimentional-file-read-to-array
;				  - UDF updated the 6 June 2012 :
;						* _ArrayDeleteLines added
;						* Samples code updated
;				  - UDF updated the 9 September 2012 :
;						* _Array2DToString updated (to work correctly with 1D array) and renamed in _ArrayToStringEx
;						* _Array2DToString corrected for a bug with Item equal to 0 and $EmptyLine flag
;						* _ArrayConcatenateEx added
;						* Samples code updated
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_StringToArray1D
;_StringToArray2D
;_ArrayToStringEx
;_Array2DToHtmlArray
;_ArrayDeleteLines
;_ArrayConcatenateEx
;_HtmlArrayToArray2D
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name...........: _StringToArray1D
; Description ...: Convert a string in 1D array (each line separate by a char or a string)
; Syntax.........: _StringToArray1D($sString[, $sDelimRow = ""[, $EmptyLine = 1[, $IndexBase = 1]]])
; Parameters ....: $sString				- String to convert
;                  $sDelimRow			- [optional] Char used for separate lines (Default @CRLF)(can be a string)
;                  $EmptyLine			- [optional] 	0 Don't keep empty lines.
;														1 Keep the empty lines (Default).
;                  $IndexBase 			- [optional]	0 Return an array that begin in index 0.
;														1 Return an array that begin in line 1 with elements numbers in index 0 (Default).
;                  $iLineStart   		- [optional] Line index to start processing
;                  $iLineEnd     		- [optional] Line index to stop processing
;
; Return values .: Success - Return an 1D array
;                  Failure - 0, sets @error:
;                  |1 - $sString is empty
;                  |2 - $sString is not correct to proceed
;                  |3 - $$iLineStart is greater than $iLineEnd
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......: _StringToArray2D
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _StringToArray1D($sString, $sDelimRow = "", $iEmptyLine = 1, $iIndexBase = 1, $iLineStart = 0, $iLineEnd = 0)
	If $sString = "" Then Return SetError(1, 0, 0)
	If $sDelimRow = "" Then $sDelimRow = @CRLF
	If $iEmptyLine <> 0 Then $iEmptyLine = 1
	If $iIndexBase <> 0 Then $iIndexBase = 1

	; Bounds checking #1
	If $iLineStart < 0 Then $iLineStart = 0
	If $iLineStart > $iLineEnd Then Return SetError(3, 0, "")


	; Remove last line separator if any at the end of the file
	If StringRight($sString, 1) = @LF Then $sString = StringTrimRight($sString, 1)
	If StringRight($sString, 1) = @CR Then $sString = StringTrimRight($sString, 1)

	; Variables declaration
	Local $avArray, $i, $Count

	; Convert the string in 1D array
	If StringInStr($sString, $sDelimRow) Then
		$avArray = StringSplit(StringStripCR($sString), StringReplace($sDelimRow, @CR, ""))
	Else
		If StringLen($sString) Then
			Dim $avArray[2] = [1, $sString]
		Else
			Return SetError(2, 0, 0)
		EndIf
	EndIf

	; Bounds checking #2
	Local $iUBound = UBound($avArray, 1) - 1
	If $iLineEnd < 1 Or $iLineEnd > $iUBound Then $iLineEnd = $iUBound

	; Keep only lines from $iLineStart to $iLineEnd
	If $iLineStart <> 0 Or $iLineEnd <> $iUBound Then
		; Prevent for a linestart greater than the dimension of the array
		If $iLineStart < $iUBound Then
			$Count = 1
			For $i = $iLineStart To $iLineEnd
				$avArray[$Count] = $avArray[$i]
				$Count += 1
			Next
			ReDim $avArray[$iLineEnd - $iLineStart + 2]
		EndIf
	EndIf

	; Delete blank line if require
	If $iEmptyLine = 0 Then
		$Count = 1
		For $i = 1 To UBound($avArray) - 1
			If $avArray[$i] <> "" Then
				$avArray[$Count] = $avArray[$i]
				$Count += 1
			EndIf
		Next
		ReDim $avArray[$Count]
		$avArray[0] = UBound($avArray) - 1
	EndIf

	; Delete array dimension in 1st element
	If $iIndexBase = 0 Then
		For $i = 0 To UBound($avArray) - 2
			$avArray[$i] = $avArray[$i + 1]
		Next
		ReDim $avArray[UBound($avArray) - 1]
	EndIf

	Return $avArray
EndFunc   ;==>_StringToArray1D

; #FUNCTION# ====================================================================================================================
; Name...........: _StringToArray2D
; Description ...: Convert a string in array 2D (each element separate by a char and each line by another char)
; Syntax.........: _StringToArray2D($sString[, $sDelimCol = ""[, $sDelimRow = ""[, $EmptyLine = 1[, $IndexBase = 1[, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0]]]]]]]])
; Parameters ....: $sString				- String to convert
;                  $sDelimRow			- [optional] Char used for separate lines (Default @CRLF)(can be a string)
;				   $sDelimCol			- [optional] Char used for separate elements (Default "|")(can be a string)
;                  $EmptyLine			- [optional] 	0 Don't keep empty lines.
;														1 Keep the empty lines (Default).
;                  $IndexBase 			- [optional]	0 Return an array that begin in index 0.
;														1 Return an array that begin in line 1 with elements numbers in index 0 (Default).
;                  $iLineStart   		- [optional] Line index to start processing
;                  $iLineEnd     		- [optional] Line index to stop processing
;                  $iColStart   		- [optional] Column index to start processing
;                  $iColEnd     		- [optional] Column index to stop processing
;
; Return values .: Success - Return an array
;                  Failure - 0, sets @error:
;                  |1 - $sString is empty
;                  |2 - $sString is not correct to proceed
;                  |3 - $$iLineStart is greater than $iLineEnd or $ColStart is greater than $ColEnd
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _StringToArray2D($sString, $sDelimRow = "", $sDelimCol = "", $iEmptyLine = 0, $iIndexBase = 0, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0)
	If $sString = "" Then Return SetError(1, 0, 0)
	If $sDelimCol = "" Then $sDelimCol = Chr(124)
	If $sDelimRow = "" Then $sDelimRow = @CRLF
	If $iEmptyLine <> 1 Then $iEmptyLine = 0
	If $iIndexBase <> 1 Then $iIndexBase = 0

	; Bounds checking #1
	If $iLineStart < 0 Then $iLineStart = 0
	If $ColStart < 0 Then $ColStart = 0
	If $iLineStart > $iLineEnd Then Return SetError(3, 0, "")
	If $ColStart > $ColEnd Then Return SetError(3, 0, "")

	; Variables declaration
	Local $aArray, $avArray, $iMaxElement = 0, $iSecondElement = 0, $aSplit

	; Get the string in array1D
	$aArray = _StringToArray1D($sString, $sDelimRow, $iEmptyLine, 1, $iLineStart, $iLineEnd)
	If @error Then
		Return SetError(@error, 0, 0)
	EndIf

	; Get the maximal dimension of the array
	For $iLinesCount = 1 To UBound($aArray) - 1
		If $aArray[$iLinesCount] <> '' Then
			$iSecondElement = UBound(StringSplit($aArray[$iLinesCount], $sDelimCol, 1) - 1) - 1
			If $iSecondElement > $iMaxElement Then $iMaxElement = $iSecondElement
		EndIf
	Next

	; Check if we want dimension in index[0]
	If $iIndexBase = 1 Then
		; Prepare the new array with all dimensions and keep [0]
		Local $avArray[UBound($aArray)][$iMaxElement]
		; Populate the new array
		For $iCount = 0 To UBound($aArray) - 1
			$aSplit = StringSplit($aArray[$iCount], $sDelimCol, 1)
			For $iCount2 = 1 To UBound($aSplit) - 1
				$avArray[$iCount][$iCount2 - 1] = $aSplit[$iCount2]
			Next
		Next
		$avArray[0][0] = UBound($avArray, 1) - 1
	Else
		; Prepare the new array with all dimensions
		Local $avArray[UBound($aArray) - 1][$iMaxElement]
		; Populate the new array
		For $iCount = 1 To UBound($aArray) - 1
			$aSplit = StringSplit($aArray[$iCount], $sDelimCol, 1)
			For $iCount2 = 1 To $aSplit[0]
				$avArray[$iCount - 1][$iCount2 - 1] = $aSplit[$iCount2]
			Next
		Next
	EndIf


	; Check if we want only some columns
	If $ColStart <> 0 And $ColEnd <> 0 Then
		; Bounds checking #2
		If $ColEnd < 1 Or $ColEnd > $iMaxElement Then $ColEnd = $iMaxElement

		; Move values and Redim the array
		For $i = $iIndexBase To UBound($avArray, 1) - 1
			Local $ColCount = 0
			For $j = $ColStart - 1 To $ColEnd - 1
				$avArray[$i][$ColCount] = $avArray[$i][$j]
				$ColCount += 1
			Next
		Next

		; Prevent negative value due to small array
		If $ColEnd - $ColStart < 1 Then $ColStart = 0
		ReDim $avArray[UBound($avArray, 1)][$ColEnd - $ColStart + 1]
	EndIf

	Return $avArray
EndFunc   ;==>_StringToArray2D

; #FUNCTION# ====================================================================================================================
; Name...........: _ArrayToStringEx
; Description ...: Convert a 1D or a 2D array 2D in a string (each element of the array will be separate by a char)
; Syntax.........: _ArrayToStringEx(Const ByRef $avArray[, $sDelimCol = ""[, $sDelimRow = @CRLF[, $EmptyLine = 1[, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0]]]]]]])
; Parameters ....: $avArray				- Array to convert (1D or 2D)
;                  $sDelimRow			- [optional] Char used for separate lines (Default @CRLF)(can be a string)
;				   $sDelimCol			- [optional] Char used for separate elements (Default "|")(can be a string)
;                  $EmptyLine			- [optional] 	0 Don't keep empty lines.
;														1 Keep the empty lines (Default).
;                  $iLineStart   		- [optional] Line index to start processing
;                  $iLineEnd     		- [optional] Line index to stop processing
;                  $iColStart   		- [optional] Column index to start processing
;                  $iColEnd     		- [optional] Column index to stop processing
;
; Return values .: Success - Return a string
;                  Failure - 0, sets @error:
;                  |1 - $avArray is not an array
;                  |2 - $avArray has too many dimensions (only up to 2D supported)
;                  |3 - $$iLineStart is greater than $iLineEnd or $ColStart is greater than $ColEnd
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......: _Array2DToHtml
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _ArrayToStringEx(Const ByRef $avArray, $sDelimRow = "", $sDelimCol = "", $iEmptyLine = 1, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0)
	If Not IsArray($avArray) Then Return SetError(1, 0, 0)
	If $sDelimCol = "" Then $sDelimCol = Chr(124)
	If $sDelimRow = "" Then $sDelimRow = @CRLF
	If $iEmptyLine <> 1 Then $iEmptyLine = 0

	; Dimension checking
	Local $iDimension = UBound($avArray, 0), $iUBound = UBound($avArray, 1) - 1, $iSubMax = UBound($avArray, 2) - 1
	If $iDimension > 2 Then Return SetError(2, 0, 0)

	; Bounds checking
	If $iLineEnd < 1 Or $iLineEnd > $iUBound Then $iLineEnd = $iUBound
	If $ColEnd < 1 Or $ColEnd > $iSubMax Then $ColEnd = $iSubMax
	If $iLineStart < 0 Then $iLineStart = 0
	If $ColStart < 0 Then $ColStart = 0
	If $iLineStart > $iLineEnd Then Return SetError(3, 0, "")

	Local $avArrayText, $vTmp
	Select
		Case $iDimension = 1 ; Case 1D Array
			For $i = $iLineStart To $iLineEnd
				; Check for blank line
				If $iEmptyLine = 1 Then
					$avArrayText &= $avArray[$i] & $sDelimRow
				ElseIf $iEmptyLine = 0 And StringLen($avArray[$i]) <> 0 Then
					$avArrayText &= $avArray[$i] & $sDelimRow
				EndIf
			Next
			Return $avArrayText

		Case $iDimension > 1 ; Case 2D Array
			If $ColStart > $ColEnd Then Return SetError(3, 0, "")
			; Convert array into text
			For $i = $iLineStart To $iLineEnd
				For $j = $ColStart To $ColEnd
					; Get current line items
					$vTmp &= $sDelimCol & $avArray[$i][$j]
				Next
				; Add line to text array
				$vTmp = StringReplace($vTmp, $sDelimCol & $sDelimCol, "", 0, 1)
				If StringRight($vTmp, 1) = $sDelimCol Then
					$vTmp = StringTrimRight($vTmp, StringLen($sDelimCol))
				EndIf
				$vTmp = StringTrimLeft($vTmp, StringLen($sDelimCol))

				; Check for blank line
				If $iEmptyLine = 1 Then
					$avArrayText &= $vTmp & $sDelimRow
				ElseIf $iEmptyLine = 0 And StringLen($vTmp) <> 0 Then
					$avArrayText &= $vTmp & $sDelimRow
				EndIf

				; Clear $vTmp for next loop
				$vTmp = ""
			Next

			Return $avArrayText
	EndSelect
EndFunc   ;==>_ArrayToStringEx

; #FUNCTION# ====================================================================================================================
; Name...........: _Array2DToArray
; Description ...: Convert a 2D array in 1D array (each element will be separate by a char or a string)
; Syntax.........: _Array2DToArray(Const ByRef $avArray[, $sDelimCol = ""[, $sDelimRow = @CRLF[, $EmptyLine = 1[, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0]]]]]]])
; Parameters ....: $avArray				- Array 2D to convert
;                  $sDelimRow			- [optional] Char used for separate lines (Default @CRLF)(can be a string)
;				   $sDelimCol			- [optional] Char used for separate elements (Default "|")(can be a string)
;                  $EmptyLine			- [optional] 	0 Don't keep empty lines.
;														1 Keep the empty lines (Default).
;                  $iLineStart   		- [optional] Line index to start processing
;                  $iLineEnd     		- [optional] Line index to stop processing
;                  $iColStart   		- [optional] Column index to start processing
;                  $iColEnd     		- [optional] Column index to stop processing
;
; Return values .: Success - Return array
;                  Failure - 0, sets @error:
;                  |1 - $avArray is not an array
;                  |2 - $avArray has too many dimensions (only up to 2D supported)
;                  |3 - $$iLineStart is greater than $iLineEnd or $ColStart is greater than $ColEnd
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _Array2DToArray(Const ByRef $avArray, $sDelimRow = "", $sDelimCol = "", $iEmptyLine = 1, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0)
	If Not IsArray($avArray) Then Return SetError(1, 0, 0)
	If $sDelimCol = "" Then $sDelimCol = Chr(124)
	If $sDelimRow = "" Then $sDelimRow = @CRLF
	If $iEmptyLine <> 1 Then $iEmptyLine = 0

	; Dimension checking
	Local $iDimension = UBound($avArray, 0), $iUBound = UBound($avArray, 1) - 1, $iSubMax = UBound($avArray, 2) - 1
	If $iDimension > 2 Then Return SetError(2, 0, 0)
	If $iDimension = 1 Then Return 1

	; Bounds checking
	If $iLineEnd < 1 Or $iLineEnd > $iUBound Then $iLineEnd = $iUBound
	If $ColEnd < 1 Or $ColEnd > $iSubMax Then $ColEnd = $iSubMax
	If $iLineStart < 0 Then $iLineStart = 0
	If $ColStart < 0 Then $ColStart = 0
	If $iLineStart > $iLineEnd Then Return SetError(3, 0, "")
	If $ColStart > $ColEnd Then Return SetError(3, 0, "")

	Local $avArray1D = $avArray
	Local $vTmp = ""
	Local $vCount = 0

	; Convert array into text
	For $i = $iLineStart To $iLineEnd
		For $j = $ColStart To $ColEnd
			; Get current line items
			$vTmp &= $sDelimCol & $avArray[$i][$j]
		Next
		; Add line to text array
		$vTmp = StringReplace($vTmp, $sDelimCol & $sDelimCol, "", 0, 1)
		If StringRight($vTmp, 1) = $sDelimCol Then
			$vTmp = StringTrimRight($vTmp, StringLen($sDelimCol))
		EndIf
		$vTmp = StringTrimLeft($vTmp, StringLen($sDelimCol))

		; Check for blank line
		If $iEmptyLine = 0 And $vTmp = "" Then
			$vCount += 1
		Else
			$avArray1D[$i - $vCount][0] = $vTmp
		EndIf

		; Clear $vTmp for next loop
		$vTmp = ""
	Next

	ReDim $avArray1D[($iLineEnd - $iLineStart) - $vCount + 1][1]
	$avArray1D[0][0] = UBound($avArray1D) - 1
	Return $avArray1D
EndFunc   ;==>_Array2DToArray

; #FUNCTION# ====================================================================================================================
; Name...........: _Array2DToHtmlArray
; Description ...: Convert a 1D or 2D array in Html array
; Syntax.........: _Array2DToHtmlArray(Const ByRef $avArray[, $sBorder = 1[, $EmptyLine = 1[, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0]]]]]])
; Parameters ....: $avArray				- Array to convert (1D or 2D)
;                  $sBorder				- [optional] Determine if the Html array have border or nor (Default = 1)
;                  $EmptyLine			- [optional] 	0 Don't keep empty lines.
;														1 Keep the empty lines (Default).
;                  $iLineStart   		- [optional] Line index to start processing
;                  $iLineEnd     		- [optional] Line index to stop processing
;                  $iColStart   		- [optional] Column index to start processing
;                  $iColEnd     		- [optional] Column index to stop processing
;
; Return values .: Success - Return a Html string
;                  Failure - 0, sets @error:
;                  |1 - $avArray is not an array
;                  |2 - $avArray has too many dimensions (only up to 2D supported)
;                  |3 - $$iLineStart is greater than $iLineEnd or $ColStart is greater than $ColEnd
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _Array2DToHtmlArray(Const ByRef $avArray, $sBorder = 1, $iEmptyLine = 1, $iLineStart = 0, $iLineEnd = 0, $ColStart = 0, $ColEnd = 0)
	If Not IsArray($avArray) Then Return SetError(1, 0, 0)
	If $sBorder <> 1 Then
		$sBorder = 'border="0"'
	Else
		$sBorder = 'border="1"'
	EndIf
	If $iEmptyLine <> 1 Then $iEmptyLine = 0

	Local $LineSpacing = '</TD>' & @CRLF & '</TR>' & @CRLF & '<TR>' & @CRLF & @TAB & '<TD>'

	Local $sHtml = _ArrayToStringEx($avArray, $LineSpacing, '</TD>' & '<TD>', $iEmptyLine, $iLineStart, $iLineEnd, $ColStart, $ColEnd)
	If @error Then
		Return SetError(@error, 0, 0)
	EndIf

	$sHtml = '<table ' & $sBorder & '>' & @CRLF & '<TR>' & @CRLF & @TAB & '<TD>' & $sHtml & '</table>' & @CRLF
	Return StringReplace($sHtml, '<TR>' & @CRLF & '<TD></table>', '</table>')
EndFunc   ;==>_Array2DToHtmlArray

; #FUNCTION# ====================================================================================================================
; Name...........: _ArrayDeleteLines
; Description ...: Delete line(s) that contain specified string on 1D or 2D arrays
; Syntax.........: _ArrayDeletelines(ByRef $avArray[, $sString = ""[, $iFlag = 1]])
; Parameters ....: $avArray				- Array to proceed (1D or 2D)
; 				   $sString				- [optional] Deletion delimiter string(s) (blanc value by default for empty line)
;													 Can be multiple value separated by semicolon ("Value1;Value2; ...")
;                  $iFlag	 			- [optional] 0 Search delimiter string in rows values
;													 1 The rows values must be equal to delimiter (Default).
;
; Return values .: Success - Return array without line(s) that contain the specified value(s)
;                  Failure - 0, sets @error:
;                  |1 - $avArray is not an array
;                  |2 - $avArray has too many dimensions (only up to 2D supported)
;
; Author ........: mikell (original function name was _ArrayDeleteEmptyRows)
; Modified.......: Tlem (tlem@tuxolem.fr) adding specified delimiter, multiple delimiters, total elements update, conventional shaping of the function
; Remarks .......:
; Related .......:
; Link ..........: Original function here : http://www.autoitscript.fr/forum/viewtopic.php?f=21&p=67660#p67660
; Example .......: Yes
; ===============================================================================================================================
Func _ArrayDeleteLines(ByRef $avArray, $sString = "", $iFlag = 1)
	If Not IsArray($avArray) Then Return SetError(1, 0, 0)
	If $iFlag <> 1 Then $iFlag = 0
	Local $aString = StringSplit($sString, ";")
	Local $fired = 0

	; Dimension checking
	Local $iDimension = UBound($avArray, 0), $iUBound = UBound($avArray, 1) - 1
	If $iDimension > 2 Then Return SetError(2, 0, 0)

	Switch $iDimension
		Case 1 ; 1D array
			Local $aArrayTmp[UBound($avArray)]
			For $i = 0 To UBound($avArray) - 1
				For $k = 1 To $aString[0]
					Switch $iFlag
						Case 0 ; Line contain value
							If StringInStr($avArray[$i], $aString[$k]) Then ; If line value contain $sString value
								$fired += 1
								ContinueLoop 2; Nextline (loop $i)
							EndIf

						Case 1 ; Exact match
							If $avArray[$i] = $aString[$k] Then ; If line value is equal to $sString value
								$fired += 1
								ContinueLoop 2 ; next line (loop $i)
							EndIf
					EndSwitch
				Next
				$aArrayTmp[$i - $fired] = $avArray[$i]
			Next
			ReDim $aArrayTmp[UBound($avArray) - $fired]

		Case 2 ; 2D array
			Local $cols = UBound($avArray, 2), $row = ""
			Local $aArrayTmp[UBound($avArray)][$cols]
			For $i = 0 To UBound($avArray) - 1
				For $k = 1 To $aString[0]
					$row = ""
					For $j = 0 To $cols - 1
						$row &= $avArray[$i][$j] ; Merge line values
					Next

					If $iFlag = 0 Then ; Line contain value
						If StringInStr($row, $aString[$k]) Then ; If row values contain $sString value
							$fired += 1
							ContinueLoop 2 ; Nextline (loop $i)
						EndIf
					Else ; Exact match
						If $row = $aString[$k] Then ; If row values is equal to $sString value
							$fired += 1
							ContinueLoop 2 ; next line (loop $i)
						EndIf
					EndIf

					For $j = 0 To $cols - 1
						$aArrayTmp[$i - $fired][$j] = $avArray[$i][$j]
					Next
				Next
			Next
			ReDim $aArrayTmp[$iUBound - $fired][$cols]
	EndSwitch

	; Check and update elements number
	If $iDimension > 1 Then
		If $aArrayTmp[0][0] = $iUBound Then
			$aArrayTmp[0][0] = UBound($aArrayTmp, 1) - 1
		EndIf
	Else
		If $aArrayTmp[0] = $iUBound Then
			$aArrayTmp[0] = UBound($aArrayTmp, 1) - 1
		EndIf
	EndIf

	$avArray = $aArrayTmp
EndFunc   ;==>_ArrayDeleteLines

; #FUNCTION# ====================================================================================================================
; Name...........: _ArrayConcatenateEx
; Description ...: Concatenate two arrays 1D or 2D.
; Syntax.........: _ArrayConcatenateEx(ByRef $avArrayTarget, Const ByRef $avArraySource, $iStartTarget = 0, $iStart = 0)
; Parameters ....: $avArrayTarget - The array to concatenate onto
;                  $avArraySource - The array to concatenate from
;                  $iStartTarget - index of the first Target Array entry
;                  $iStartSource - index of the first Source Array entry
; Return values .: Success - The new array
;                  Failure - 0, sets @error to:
;                  |1 - $avArrayTarget is not an array
;                  |2 - $avArraySource is not an array
;                  |3 - $avArraySource or $avArraySource wrong dimension
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......: 2D + 2D code inspired by PandiPanda (http://autoitscript.fr/forum/viewtopic.php?p=71689#p71689)
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _ArrayConcatenateEx(ByRef $avArrayTarget, Const ByRef $avArraySource, $iStart = 0)
	If Not IsArray($avArrayTarget) Then Return SetError(1, 0, 0)
	If Not IsArray($avArraySource) Then Return SetError(2, 0, 0)

	; Dimension checking
	If UBound($avArrayTarget, 0) > 2 Or UBound($avArraySource, 0) > 2 Then Return SetError(3, 0, 0)

	Local $iUBoundTarget, $iUBoundSource, $i, $j

	Select
		; Case of two array 2D
		Case UBound($avArrayTarget, 0) = 2 And UBound($avArraySource, 0) = 2
			$iUBoundTarget = UBound($avArrayTarget) - $iStart
			$iUBoundSource = UBound($avArraySource)
			Local $iUBoundColTarget = UBound($avArrayTarget, 2), $iUBoundColSource = UBound($avArraySource, 2)

			If $iUBoundColTarget < $iUBoundColSource Then
				ReDim $avArrayTarget[$iUBoundTarget + $iUBoundSource][$iUBoundColSource]
			Else
				ReDim $avArrayTarget[$iUBoundTarget + $iUBoundSource][$iUBoundColTarget]
			EndIf

			For $i = $iStart To UBound($avArraySource, 1) - 1
				For $j = 0 To UBound($avArraySource, 2) - 1
					$avArrayTarget[$i + $iUBoundTarget][$j] = $avArraySource[$i][$j]
				Next
			Next
			Return $iUBoundTarget + $iUBoundSource

		; Case of two array 1D
		Case UBound($avArrayTarget, 0) = 1 And UBound($avArraySource, 0) = 1
			$iUBoundTarget = UBound($avArrayTarget) - $iStart
			$iUBoundSource = UBound($avArraySource)

			ReDim $avArrayTarget[$iUBoundTarget + $iUBoundSource]
			For $i = $iStart To UBound($avArraySource, 1) - 1
				$avArrayTarget[$i + $iUBoundTarget] = $avArraySource[$i]
			Next
			Return $iUBoundTarget + $iUBoundSource

		; Case of 1D array + 2D array (3x slower than previous code)
		Case Else
			Local $sArrayTarget = _ArrayToStringEx($avArrayTarget)
			Local $sArraySource = _ArrayToStringEx($avArraySource, "", "", 1, $iStart)
			$avArrayTarget = _StringToArray2D($sArrayTarget & @CRLF & $sArraySource)
			Return UBound($avArrayTarget)
	EndSelect
EndFunc   ;==>_ArrayConcatenateEx

; #FUNCTION# ====================================================================================================================
; Name...........: _HtmlArrayToArray2D
; Description ...: Get a 1D or a 2D array from Url or Html file
; Syntax.........: _HtmlArrayToArray2D($sHTML = ""[, $iIndex = 0[, $iTranspose = 1]])
; Parameters ....: $sHTML				- Url or File name and path
;                  $iIndex				- [optional] Index of the array to retrieve (Default = 0)
;                  $iTranspose			- [optional] 	0 Don't transpose the array.
;														1 Transpose the array (Default).
;
; Return values .: On Success	- Returns an array
;                  On Failure	- Returns 0 and sets @ERROR
;					@ERROR		- -1 = $sHTML is empty string
;								- 0 ($_IEStatus_Success) = No Error
;								- 1 ($_IEStatus_GeneralError) = General Error
;								- 3 ($_IEStatus_InvalidDataType) = Invalid Data Type
;								- 4 ($_IEStatus_InvalidObjectType) = Invalid Object Type
;								- 5 ($_IEStatus_InvalidValue) = Invalid Value
;								- 6 ($_IEStatus_LoadWaitTimeout) = Load Wait Timeout
;								- 7 ($_IEStatus_NoMatch) = No Match
;								- 8 ($_IEStatus_AccessIsDenied) = Access Is Denied
;								- 9 ($_IEStatus_ClientDisconnected) = Client Disconnected
;					@Extended	- Set to true (1) or false (0) depending on the success of $f_tryAttach
;
; Author ........: Tlem (tlem@tuxolem.fr)
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _HtmlArrayToArray2D($sHtml = "", $iIndex = 0, $iTranspose = 1)
	If $sHtml = "" Then Return SetError(@error, 1, 0)
	If $iIndex < 0 Or Not IsNumber($iIndex) Then $iIndex = 0 ; Set $iIndex to 0 if < 0 or not a number
	If $iTranspose <> 0 And $iTranspose <> 1 Then $iTranspose = 1 ; Set $iTranspose = 1 if <> 0 or 1

	Local $fHTML, $oIE, $oTable, $aTableData

	$fHTML = FileRead($sHtml)
	If @error Then
		; $sHTML is probably a web link
		$oIE = _IECreate($sHtml, 0, 0, 1)
		If @error Then
			Return SetError(@error, 1, 0)
		EndIf
	Else
		; $sHTML is a file
		$oIE = _IECreate("about:blank", 0, 0, 1)
		If @error Then
			Return SetError(@error, 1, 0)
		EndIf
		_IEDocWriteHTML($oIE, $fHTML)
		If @error Then
			_IEQuit($oIE)
			Return SetError(@error, 1, 0)
		EndIf
	EndIf

	; Get table collection by index
	$oTable = _IETableGetCollection($oIE, $iIndex)
	If @error Then
		_IEQuit($oIE)
		Return SetError(@error, 1, 0)
	EndIf

	; Reads the contents of the Table into an array
	$aTableData = _IETableWriteToArray($oTable, $iTranspose)
	If @error Then
		_IEQuit($oIE)
		Return SetError(@error, 1, 0)
	EndIf

	_IEQuit($oIE)
	Return $aTableData
EndFunc   ;==>_HtmlArrayToArray2D
