#include "ArrayEx.au3"
#include <array.au3>

#Region ### Creation of example files ###
Global $aFile[6] = ["Array_1.txt", "Array_2.txt", "Array_3.txt", "Array_4.txt", "Array.html"]
_Complex_Array_File(@ScriptDir & "\" & $aFile[0])
_Big_Array_File(@ScriptDir & "\" & $aFile[1])
_Simple_Array_File(@ScriptDir & "\" & $aFile[2])
_Medium_Array_File(@ScriptDir & "\" & $aFile[3])
_HTML_Array_File(@ScriptDir & "\" & $aFile[4])
; Test if all files have been created.
For $i = 0 To UBound($aFile) - 1
	If Not FileExists(@ScriptDir & "\" & $aFile[$i]) Then _Error()
Next
#EndRegion ### END Creation of exemple files ###

Global $aString, $File, $String

MsgBox(64, "Information", "This is the Array_1.txt example file" & @CRLF & @CRLF & FileRead("Array_1.txt"))

; _StringToArray1D : Read the file and create an array 1D.
$File = FileRead("Array_1.txt")
$aString = _StringToArray1D($File)
_ArrayDisplay($aString, "Array 1D from file or string")

; _StringToArray2D : Read the file and create an array 2D.
$File = FileRead("Array_1.txt")
$aString = _StringToArray2D($File, @CRLF, ",")
_ArrayDisplay($aString, "Array 2D from file or string")

; _StringToArray2D : Read the file and create an array 2D without blanc line and without index.
$File = FileRead("Array_1.txt")
$aString = _StringToArray2D($File, @CRLF, ",", 0, 0)
_ArrayDisplay($aString, "Array 2D from file or string No index and blank line")

; _StringToArray2D : Read file Array_2.txt and create a new array from line 1 to 20 and row 10 to 20
$File = FileRead("Array_2.txt")
$aString = _StringToArray2D($File, @CRLF, ",", 1, 1, 1, 20, 10, 20)
_ArrayDisplay($aString, "Line 1 to 20 and row 10 to 20 from file Array_2.txt")
; Note that in this exemple, the array point a disturbing behaviour for some persons.
; I ask to read from line 1 to 20 and columns 10 to 20.
; The array have 20 Lines ([1] to [20]) but 11 Columns ([0] to [10]), because for lines I start to 1
; and for columns, I start from 10 (it's like if I start from 0).

; _Array2DToString : Now convert array to string
$String = _ArrayToStringEx($aString, "", ",", 1, 1)
MsgBox(64, "Information", "Array2D converted in string" & @TAB & @CRLF & @CRLF & $String)

; _ArrayDeleteLines : Delete all empty lines of a 50X10 array and then delete all lines that contains value 10 ou 100.
$File = FileRead("Array_4.txt")
$aString = _StringToArray2D($File, @CRLF, ",", 1)
_ArrayDisplay($aString, "Array 50X10")
_ArrayDeleteLines($aString)
_ArrayDisplay($aString, "Empty lines of the array have been deleted")
_ArrayDeleteLines($aString, "10;100", 0)
_ArrayDisplay($aString, "All line that contains value 10 and 100 have been deleted")

; _ArrayConcatenateEx Add $aArray2 (2D) to $aArray1 (2D) :
Dim $aArray1[4][3]=[["a","b","1"],["c","d","2"],["e","f","3"],["g","h","4"]]
Dim $aArray2[3][5]=[["i","j","5","6","7"],["k","k","6","7","8"],["m","n","7","8","9"]]
_ArrayConcatenateEx($aArray1, $aArray2)
If @error Then MsgBox(0,"", "Error N" & @error)
_ArrayDisplay($aArray1, "$aArray1 + $aArray2")

; _ArrayConcatenateEx Add aArray3 (1D) to new aArray1 (2D) :
Dim $aArray3[4]=["a1","b1","c1","d1"]
_ArrayConcatenateEx($aArray1, $aArray3)
If @error Then MsgBox(0,"", "Error N" & @error)
_ArrayDisplay($aArray1, "New $aArray1 + $aArray3")

; _HtmlArrayToArray2D : Read second array from html file
$File = @ScriptDir & "\Array.html"
$Index = 1
MsgBox(64, "Information", "For next exemple, clic OK " & "and wait 5 or 10 seconds")
$aArray = _HtmlArrayToArray2D($File, $Index)
_ArrayDisplay($aArray, "Second array from Array.html file")

; _HtmlArrayToArray2D : Read array from URL
$File = "http://www.autoitscript.com/trac/autoit/query?status=new"
$Index = 2
MsgBox(64, "Information", "For next exemple, clic OK " & "and wait 5 or 10 seconds")
$aArray = _HtmlArrayToArray2D($File, $Index)
_ArrayDisplay($aArray, "Array from URL")

; _Array2DToHtmlArray : Create html array from array2D
$File = @ScriptDir & "\Exemple.html"
$String = _Array2DToHtmlArray($aArray, 1, 0)
$html = "<html>" & @CRLF & $String & "</html>"
$hFile = FileOpen($File, 2)
FileWrite($hFile, $html)
FileClose($hFile)
ShellExecuteWait($File)

MsgBox(64, "ArrayEx examples", "End of all the samples of use of ArrayEx UDF." & @CRLF & "Examples files will be deleted now")
FileDelete(@ScriptDir & "\*.txt")
FileDelete(@ScriptDir & "\*.html")
Exit


#Region ### Functions for example files creation ###
Func _Complex_Array_File($sFile)
	Global $i, $j, $Var
	Global $hFile = FileOpen($sFile, 2)
	If $hFile = -1 Then Return SetError(1, 0, 0)

	; Create a 10X10 array
	For $i = 1 To 10
		$Var = ""
		For $j = 1 To 10
			$Var &= Random(1, 100, 1) & ","
		Next
		FileWriteLine($hFile, StringTrimRight($Var, 1))
	Next
	FileWriteLine($hFile, "") ; Add a blank line
	; Add a 8X15 array
	For $i = 1 To 8
		$Var = ""
		For $j = 1 To 15
			$Var &= Random(1, 100, 1) & ","
		Next
		FileWriteLine($hFile, StringTrimRight($Var, 1))
	Next

	FileClose($hFile)
	Return 1
EndFunc   ;==>_Complex_Array_File

Func _Medium_Array_File($sFile)
	Global $i, $j, $Var
	Global $hFile = FileOpen($sFile, 2)
	If $hFile = -1 Then Return SetError(1, 0, 0)

	; Create a 50X10 array with empty lines
	For $i = 1 To 50
		$Var = ""
		If Random(1, 4, 1) = 1 Then
			FileWriteLine($hFile, "")
		Else
			For $j = 1 To 10
				$Var &= Random(1, 100, 1) & ","
			Next
			FileWriteLine($hFile, StringTrimRight($Var, 1))
		EndIf
	Next

	FileClose($hFile)
	Return 1
EndFunc   ;==>_Big_Array_File

Func _Big_Array_File($sFile)
	Global $i, $j, $Var
	Global $hFile = FileOpen($sFile, 2)
	If $hFile = -1 Then Return SetError(1, 0, 0)

	; Create a 1000X200 array
	For $i = 1 To 1000
		$Var = ""
		For $j = 1 To 200
			$Var &= Random(1, 100, 1) & ","
		Next
		FileWriteLine($hFile, StringTrimRight($Var, 1))
	Next

	FileClose($hFile)
	Return 1
EndFunc   ;==>_Big_Array_File

Func _Simple_Array_File($sFile)
	Global $hFile = FileOpen($sFile, 2)
	If $hFile = -1 Then Return SetError(1, 0, 0)

	; Create only one ligne of 2 data
	FileWriteLine($hFile, Random(1, 100, 1) & "," & Random(1, 100, 1))

	FileClose($hFile)
	Return 1
EndFunc   ;==>_Simple_Array_File

Func _HTML_Array_File($sFile)
	Global $hFile = FileOpen($sFile, 2)
	If $hFile = -1 Then Return SetError(1, 0, 0)

	; Create an html file that contain 2 arrays
	$HtmlFile = '<HTML>'  & @CRLF & _
				'<TABLE frame="border" cellpadding="2">' & @CRLF & _
				'<TR>' & @CRLF & _
				'<TD> cell 1.1 </TD>' & @CRLF & _
				'   <TD> cell 1.2 </TD>' & @CRLF & _
				'  </TR>' & @CRLF & _
				'  <TR>' & @CRLF & _
				'   <TD> cell 2.1 </TD>' & @CRLF & _
				'   <TD> cell 2.2 </TD>' & @CRLF & _
				'  </TR>' & @CRLF & _
				' </TABLE>' & @CRLF & _
				'<br>' & @CRLF & _
				'<br>' & @CRLF & _
				' <TABLE frame="border" cellpadding="2">' & @CRLF & _
				'  <TR>' & @CRLF & _
				'   <TH align="center"> title 1</TH>' & @CRLF & _
				'   <TH align="center"> title 2 </TH>' & @CRLF & _
				'  </TR>' & @CRLF & _
				'  <TR>' & @CRLF & _
				'   <TD> text 1</TD>' & @CRLF & _
				'   <TD> text 2</TD>' & @CRLF & _
				'  </TR>' & @CRLF & _
				'</TABLE>' & @CRLF & _
				'</HTML>'
	FileWriteLine($hFile, $HtmlFile)

	FileClose($hFile)
	Return 1
EndFunc   ;==>_HTML_Array_File

Func _Error()
	MsgBox(16, 'Error', 'Test Files creation error. Exiting ...')
	Exit
EndFunc   ;==>_Error
#EndRegion ### END Functions for exemple files creation ###
